module Abbr where

import Data.Generics
import Core
import Traversal

--------------------------------------------------------------------------------
-- Abbreviations
--------------------------------------------------------------------------------

-- Naming convention: any part except the second, which is the 'verb', can be
-- omitted. When a part is missing, the default value is taken. Therefore, the
-- default value never shows up in a name.

-- {Full*, Once, Spine, Stop}        : prefix coverage
-- {Transform, Accumulate, AccTrans} : verb
-- {Ctx*, NoCtx}                     : suffix context
-- {TD*, BU}                         : suffix vertical order
-- {L*, R}                           : suffix horizontal order

-- Full
transform :: Data t => [GenT c a] -> t -> Maybe t
transform = traverse Trans NoCtx Full FromTop FromLeft

transform' :: Data t => [GenT c a] -> t -> Maybe t
transform' = traverse Trans NoCtx Full FromTop FromRight

transformB :: Data t => [GenT c a] -> t -> Maybe t
transformB = traverse Trans NoCtx Full FromBottom FromLeft

transformB' :: Data t => [GenT c a] -> t -> Maybe t
transformB' = traverse Trans NoCtx Full FromBottom FromRight

transformC :: Data t => [GenU c a] -> [GenCT c a] -> c -> t -> Maybe t
transformC = traverse Trans Ctx Full FromTop FromLeft

transformC' :: Data t => [GenU c a] -> [GenCT c a] -> c -> t -> Maybe t
transformC' = traverse Trans Ctx Full FromTop FromRight

transformCB :: Data t => [GenU c a] -> [GenCT c a] -> c -> t -> Maybe t
transformCB = traverse Trans Ctx Full FromBottom FromLeft

transformCB' :: Data t => [GenU c a] -> [GenCT c a] -> c -> t -> Maybe t
transformCB' = traverse Trans Ctx Full FromBottom FromRight

accumulate :: Data t => [GenA c a] -> a -> t -> Maybe a
accumulate = traverse Accum NoCtx Full FromTop FromLeft

accumulate' :: Data t => [GenA c a] -> a -> t -> Maybe a
accumulate' = traverse Accum NoCtx Full FromTop FromRight

accumulateB :: Data t => [GenA c a] -> a -> t -> Maybe a
accumulateB = traverse Accum NoCtx Full FromBottom FromLeft

accumulateB' :: Data t => [GenA c a] -> a -> t -> Maybe a
accumulateB' = traverse Accum NoCtx Full FromBottom FromRight

accumulateC :: Data t => [GenU c a] -> [GenCA c a] -> c -> a -> t -> Maybe a
accumulateC = traverse Accum Ctx Full FromTop FromLeft

accumulateC' :: Data t => [GenU c a] -> [GenCA c a] -> c -> a -> t -> Maybe a
accumulateC' = traverse Accum Ctx Full FromTop FromRight

accumulateCB :: Data t => [GenU c a] -> [GenCA c a] -> c -> a -> t -> Maybe a
accumulateCB = traverse Accum Ctx Full FromBottom FromLeft

accumulateCB' :: Data t => [GenU c a] -> [GenCA c a] -> c -> a -> t -> Maybe a
accumulateCB' = traverse Accum Ctx Full FromBottom FromRight

acctrans :: Data t => [GenAT c a] -> a -> t -> Maybe (a,t)
acctrans = traverse AccTrans NoCtx Full FromTop FromLeft

acctrans' :: Data t => [GenAT c a] -> a -> t -> Maybe (a,t)
acctrans' = traverse AccTrans NoCtx Full FromTop FromRight

acctransB :: Data t => [GenAT c a] -> a -> t -> Maybe (a,t)
acctransB = traverse AccTrans NoCtx Full FromBottom FromLeft

acctransB' :: Data t => [GenAT c a] -> a -> t -> Maybe (a,t)
acctransB' = traverse AccTrans NoCtx Full FromBottom FromRight

acctransC :: Data t => [GenU c a] -> [GenCAT c a] -> c -> a -> t -> Maybe (a,t)
acctransC = traverse AccTrans Ctx Full FromTop FromLeft

acctransC' :: Data t => [GenU c a] -> [GenCAT c a] -> c -> a -> t -> Maybe (a,t)
acctransC' = traverse AccTrans Ctx Full FromTop FromRight

acctransCB :: Data t => [GenU c a] -> [GenCAT c a] -> c -> a -> t -> Maybe (a,t)
acctransCB = traverse AccTrans Ctx Full FromBottom FromLeft

acctransCB' :: Data t => [GenU c a] -> [GenCAT c a] -> c -> a -> t -> Maybe (a,t)
acctransCB' = traverse AccTrans Ctx Full FromBottom FromRight

-- Once
onceTransform :: Data t => [GenT c a] -> t -> Maybe t
onceTransform = traverse Trans NoCtx Once FromTop FromLeft

onceTransform' :: Data t => [GenT c a] -> t -> Maybe t
onceTransform' = traverse Trans NoCtx Once FromTop FromRight

onceTransformB :: Data t => [GenT c a] -> t -> Maybe t
onceTransformB = traverse Trans NoCtx Once FromBottom FromLeft

onceTransformB' :: Data t => [GenT c a] -> t -> Maybe t
onceTransformB' = traverse Trans NoCtx Once FromBottom FromRight

onceTransformC :: Data t => [GenU c a] -> [GenCT c a] -> c -> t -> Maybe t
onceTransformC = traverse Trans Ctx Once FromTop FromLeft

onceTransformC' :: Data t => [GenU c a] -> [GenCT c a] -> c -> t -> Maybe t
onceTransformC' = traverse Trans Ctx Once FromTop FromRight

onceTransformCB :: Data t => [GenU c a] -> [GenCT c a] -> c -> t -> Maybe t
onceTransformCB = traverse Trans Ctx Once FromBottom FromLeft

onceTransformCB' :: Data t => [GenU c a] -> [GenCT c a] -> c -> t -> Maybe t
onceTransformCB' = traverse Trans Ctx Once FromBottom FromRight

onceAccumulate :: Data t => [GenA c a] -> a -> t -> Maybe a
onceAccumulate = traverse Accum NoCtx Once FromTop FromLeft

onceAccumulate' :: Data t => [GenA c a] -> a -> t -> Maybe a
onceAccumulate' = traverse Accum NoCtx Once FromTop FromRight

onceAccumulateB :: Data t => [GenA c a] -> a -> t -> Maybe a
onceAccumulateB = traverse Accum NoCtx Once FromBottom FromLeft

onceAccumulateB' :: Data t => [GenA c a] -> a -> t -> Maybe a
onceAccumulateB' = traverse Accum NoCtx Once FromBottom FromRight

onceAccumulateC :: Data t => [GenU c a] -> [GenCA c a] -> c -> a -> t -> Maybe a
onceAccumulateC = traverse Accum Ctx Once FromTop FromLeft

onceAccumulateC' :: Data t => [GenU c a] -> [GenCA c a] -> c -> a -> t -> Maybe a
onceAccumulateC' = traverse Accum Ctx Once FromTop FromRight

onceAccumulateCB :: Data t => [GenU c a] -> [GenCA c a] -> c -> a -> t -> Maybe a
onceAccumulateCB = traverse Accum Ctx Once FromBottom FromLeft

onceAccumulateCB' :: Data t => [GenU c a] -> [GenCA c a] -> c -> a -> t -> Maybe a
onceAccumulateCB' = traverse Accum Ctx Once FromBottom FromRight

onceAcctrans :: Data t => [GenAT c a] -> a -> t -> Maybe (a,t)
onceAcctrans = traverse AccTrans NoCtx Once FromTop FromLeft

onceAcctrans' :: Data t => [GenAT c a] -> a -> t -> Maybe (a,t)
onceAcctrans' = traverse AccTrans NoCtx Once FromTop FromRight

onceAcctransB :: Data t => [GenAT c a] -> a -> t -> Maybe (a,t)
onceAcctransB = traverse AccTrans NoCtx Once FromBottom FromLeft

onceAcctransB' :: Data t => [GenAT c a] -> a -> t -> Maybe (a,t)
onceAcctransB' = traverse AccTrans NoCtx Once FromBottom FromRight

onceAcctransC :: Data t => [GenU c a] -> [GenCAT c a] -> c -> a -> t -> Maybe (a,t)
onceAcctransC = traverse AccTrans Ctx Once FromTop FromLeft

onceAcctransC' :: Data t => [GenU c a] -> [GenCAT c a] -> c -> a -> t -> Maybe (a,t)
onceAcctransC' = traverse AccTrans Ctx Once FromTop FromRight

onceAcctransCB :: Data t => [GenU c a] -> [GenCAT c a] -> c -> a -> t -> Maybe (a,t)
onceAcctransCB = traverse AccTrans Ctx Once FromBottom FromLeft

onceAcctransCB' :: Data t => [GenU c a] -> [GenCAT c a] -> c -> a -> t -> Maybe (a,t)
onceAcctransCB' = traverse AccTrans Ctx Once FromBottom FromRight

-- Spine
spineTransform :: Data t => [GenT c a] -> t -> Maybe t
spineTransform = traverse Trans NoCtx Spine FromTop FromLeft

spineTransform' :: Data t => [GenT c a] -> t -> Maybe t
spineTransform' = traverse Trans NoCtx Spine FromTop FromRight

spineTransformB :: Data t => [GenT c a] -> t -> Maybe t
spineTransformB = traverse Trans NoCtx Spine FromBottom FromLeft

spineTransformB' :: Data t => [GenT c a] -> t -> Maybe t
spineTransformB' = traverse Trans NoCtx Spine FromBottom FromRight

spineTransformC :: Data t => [GenU c a] -> [GenCT c a] -> c -> t -> Maybe t
spineTransformC = traverse Trans Ctx Spine FromTop FromLeft

spineTransformC' :: Data t => [GenU c a] -> [GenCT c a] -> c -> t -> Maybe t
spineTransformC' = traverse Trans Ctx Spine FromTop FromRight

spineTransformCB :: Data t => [GenU c a] -> [GenCT c a] -> c -> t -> Maybe t
spineTransformCB = traverse Trans Ctx Spine FromBottom FromLeft

spineTransformCB' :: Data t => [GenU c a] -> [GenCT c a] -> c -> t -> Maybe t
spineTransformCB' = traverse Trans Ctx Spine FromBottom FromRight

spineAccumulate :: Data t => [GenA c a] -> a -> t -> Maybe a
spineAccumulate = traverse Accum NoCtx Spine FromTop FromLeft

spineAccumulate' :: Data t => [GenA c a] -> a -> t -> Maybe a
spineAccumulate' = traverse Accum NoCtx Spine FromTop FromRight

spineAccumulateB :: Data t => [GenA c a] -> a -> t -> Maybe a
spineAccumulateB = traverse Accum NoCtx Spine FromBottom FromLeft

spineAccumulateB' :: Data t => [GenA c a] -> a -> t -> Maybe a
spineAccumulateB' = traverse Accum NoCtx Spine FromBottom FromRight

spineAccumulateC :: Data t => [GenU c a] -> [GenCA c a] -> c -> a -> t -> Maybe a
spineAccumulateC = traverse Accum Ctx Spine FromTop FromLeft

spineAccumulateC' :: Data t => [GenU c a] -> [GenCA c a] -> c -> a -> t -> Maybe a
spineAccumulateC' = traverse Accum Ctx Spine FromTop FromRight

spineAccumulateCB :: Data t => [GenU c a] -> [GenCA c a] -> c -> a -> t -> Maybe a
spineAccumulateCB = traverse Accum Ctx Spine FromBottom FromLeft

spineAccumulateCB' :: Data t => [GenU c a] -> [GenCA c a] -> c -> a -> t -> Maybe a
spineAccumulateCB' = traverse Accum Ctx Spine FromBottom FromRight

spineAcctrans :: Data t => [GenAT c a] -> a -> t -> Maybe (a,t)
spineAcctrans = traverse AccTrans NoCtx Spine FromTop FromLeft

spineAcctrans' :: Data t => [GenAT c a] -> a -> t -> Maybe (a,t)
spineAcctrans' = traverse AccTrans NoCtx Spine FromTop FromRight

spineAcctransB :: Data t => [GenAT c a] -> a -> t -> Maybe (a,t)
spineAcctransB = traverse AccTrans NoCtx Spine FromBottom FromLeft

spineAcctransB' :: Data t => [GenAT c a] -> a -> t -> Maybe (a,t)
spineAcctransB' = traverse AccTrans NoCtx Spine FromBottom FromRight

spineAcctransC :: Data t => [GenU c a] -> [GenCAT c a] -> c -> a -> t -> Maybe (a,t)
spineAcctransC = traverse AccTrans Ctx Spine FromTop FromLeft

spineAcctransC' :: Data t => [GenU c a] -> [GenCAT c a] -> c -> a -> t -> Maybe (a,t)
spineAcctransC' = traverse AccTrans Ctx Spine FromTop FromRight

spineAcctransCB :: Data t => [GenU c a] -> [GenCAT c a] -> c -> a -> t -> Maybe (a,t)
spineAcctransCB = traverse AccTrans Ctx Spine FromBottom FromLeft

spineAcctransCB' :: Data t => [GenU c a] -> [GenCAT c a] -> c -> a -> t -> Maybe (a,t)
spineAcctransCB' = traverse AccTrans Ctx Spine FromBottom FromRight

-- Stop
stopTransform :: Data t => [GenT c a] -> t -> Maybe t
stopTransform = traverse Trans NoCtx Stop FromTop FromLeft

stopTransform' :: Data t => [GenT c a] -> t -> Maybe t
stopTransform' = traverse Trans NoCtx Stop FromTop FromRight

stopTransformB :: Data t => [GenT c a] -> t -> Maybe t
stopTransformB = traverse Trans NoCtx Stop FromBottom FromLeft

stopTransformB' :: Data t => [GenT c a] -> t -> Maybe t
stopTransformB' = traverse Trans NoCtx Stop FromBottom FromRight

stopTransformC :: Data t => [GenU c a] -> [GenCT c a] -> c -> t -> Maybe t
stopTransformC = traverse Trans Ctx Stop FromTop FromLeft

stopTransformC' :: Data t => [GenU c a] -> [GenCT c a] -> c -> t -> Maybe t
stopTransformC' = traverse Trans Ctx Stop FromTop FromRight

stopTransformCB :: Data t => [GenU c a] -> [GenCT c a] -> c -> t -> Maybe t
stopTransformCB = traverse Trans Ctx Stop FromBottom FromLeft

stopTransformCB' :: Data t => [GenU c a] -> [GenCT c a] -> c -> t -> Maybe t
stopTransformCB' = traverse Trans Ctx Stop FromBottom FromRight

stopAccumulate :: Data t => [GenA c a] -> a -> t -> Maybe a
stopAccumulate = traverse Accum NoCtx Stop FromTop FromLeft

stopAccumulate' :: Data t => [GenA c a] -> a -> t -> Maybe a
stopAccumulate' = traverse Accum NoCtx Stop FromTop FromRight

stopAccumulateB :: Data t => [GenA c a] -> a -> t -> Maybe a
stopAccumulateB = traverse Accum NoCtx Stop FromBottom FromLeft

stopAccumulateB' :: Data t => [GenA c a] -> a -> t -> Maybe a
stopAccumulateB' = traverse Accum NoCtx Stop FromBottom FromRight

stopAccumulateC :: Data t => [GenU c a] -> [GenCA c a] -> c -> a -> t -> Maybe a
stopAccumulateC = traverse Accum Ctx Stop FromTop FromLeft

stopAccumulateC' :: Data t => [GenU c a] -> [GenCA c a] -> c -> a -> t -> Maybe a
stopAccumulateC' = traverse Accum Ctx Stop FromTop FromRight

stopAccumulateCB :: Data t => [GenU c a] -> [GenCA c a] -> c -> a -> t -> Maybe a
stopAccumulateCB = traverse Accum Ctx Stop FromBottom FromLeft

stopAccumulateCB' :: Data t => [GenU c a] -> [GenCA c a] -> c -> a -> t -> Maybe a
stopAccumulateCB' = traverse Accum Ctx Stop FromBottom FromRight

stopAcctrans :: Data t => [GenAT c a] -> a -> t -> Maybe (a,t)
stopAcctrans = traverse AccTrans NoCtx Stop FromTop FromLeft

stopAcctrans' :: Data t => [GenAT c a] -> a -> t -> Maybe (a,t)
stopAcctrans' = traverse AccTrans NoCtx Stop FromTop FromRight

stopAcctransB :: Data t => [GenAT c a] -> a -> t -> Maybe (a,t)
stopAcctransB = traverse AccTrans NoCtx Stop FromBottom FromLeft

stopAcctransB' :: Data t => [GenAT c a] -> a -> t -> Maybe (a,t)
stopAcctransB' = traverse AccTrans NoCtx Stop FromBottom FromRight

stopAcctransC :: Data t => [GenU c a] -> [GenCAT c a] -> c -> a -> t -> Maybe (a,t)
stopAcctransC = traverse AccTrans Ctx Stop FromTop FromLeft

stopAcctransC' :: Data t => [GenU c a] -> [GenCAT c a] -> c -> a -> t -> Maybe (a,t)
stopAcctransC' = traverse AccTrans Ctx Stop FromTop FromRight

stopAcctransCB :: Data t => [GenU c a] -> [GenCAT c a] -> c -> a -> t -> Maybe (a,t)
stopAcctransCB = traverse AccTrans Ctx Stop FromBottom FromLeft

stopAcctransCB' :: Data t => [GenU c a] -> [GenCAT c a] -> c -> a -> t -> Maybe (a,t)
stopAcctransCB' = traverse AccTrans Ctx Stop FromBottom FromRight
